/***********************************************************************************
* Intermediate Query Development
* Dave Valentine
* Aggregates
***********************************************************************************/

USE AdventureWorks2012;
GO

/*****************************************************************************
Simple Aggregate Functions
*****************************************************************************/
SELECT SUM(SOD.orderqty)
FROM   sales.salesorderdetail SOD
INNER JOIN production.product P ON SOD.productid = P.productid

SELECT COUNT(1)
FROM   sales.salesorderdetail SOD
INNER JOIN production.product P ON SOD.productid = P.productid



/*****************************************************************************
This query will return the SUM of all order quanties, grouped by product id
and product name.
*****************************************************************************/
SELECT p.productid,
       p.Name,
       SUM(SOD.orderqty)
FROM   sales.salesorderdetail SOD
INNER JOIN production.product P ON SOD.productid = P.productid
GROUP  BY p.productid,
          p.Name


/*****************************************************************************
This query will return the Average of all order quanties, grouped by product id
and product name.  Notice the data type
*****************************************************************************/
SELECT p.productid,
       p.Name,
       AVG(SOD.orderqty)
FROM   sales.salesorderdetail SOD
INNER JOIN production.product P ON SOD.productid = P.productid
GROUP  BY p.productid,
          p.Name

-- Updated to cast the order quantity as a floating point number.
SELECT p.productid,
       p.Name,
       AVG(CAST(SOD.orderqty AS FLOAT))
FROM   sales.salesorderdetail SOD
INNER JOIN production.product P ON SOD.productid = P.productid
GROUP  BY p.productid,
          p.Name


/*****************************************************************************
This query will return multiple aggreages based on the order quanties, 
grouped by product id and product name.  What column is what?
*****************************************************************************/
SELECT p.productid,
       p.Name,
       Min(SOD.orderqty),
       Max(SOD.orderqty),
       SUM(SOD.orderqty),
	   COUNT(SOD.orderqty),
       AVG(CAST(SOD.orderqty AS FLOAT))
FROM   sales.salesorderdetail SOD
       INNER JOIN production.product P ON SOD.productid = P.productid
GROUP  BY p.productid,
          p.Name


/*****************************************************************************
This query will return multiple aggreages based on the order quanties, 
grouped by product id and product name.  However, this time we have named
the columns, which makes the result set meaningful.  It also makes the query 
easier to understand the purpose.
*****************************************************************************/
SELECT p.productid,
       p.Name,
       MinOrderQty = Min(SOD.orderqty),
       MaxOrderQty = Max(SOD.orderqty),
       SumOrderQty = SUM(SOD.orderqty),
	   NumberOrderQty = COUNT(SOD.orderqty),
       AvgOrderQty = AVG(CAST(SOD.orderqty AS FLOAT))
FROM   sales.salesorderdetail SOD
INNER JOIN production.product P ON SOD.productid = P.productid
GROUP  BY p.productid,
          p.Name 